"use client";

import FormBuilder from "@/components/FormBuilder/form-builder";
import InputField from "@/components/FormBuilder/FormFields/input-field";
import useFormBuilder from "@/components/FormBuilder/hooks/use-form-builder";
import prepareFields from "@/components/FormBuilder/utils/prepare-fields";
import Yup from "@/components/FormBuilder/utils/yup";
import FormButtons from "@/components/simulation/common/form-buttons";
import { Button } from "@/components/ui/button";
import {
	Dialog,
	DialogContent,
	DialogHeader,
	DialogTitle,
	DialogTrigger,
} from "@/components/ui/dialog";
import { Map } from "@/components/ui/map";
import PhoneInput from "@/components/ui/phone-input";
import {
	Select,
	SelectContent,
	SelectItem,
	SelectTrigger,
	SelectValue,
} from "@/components/ui/select";
import { useRouter } from "@/i18n/routing";
import { cn } from "@/lib/utils";
import { fetchArea, fetchCity } from "@/servers/actions/city-area-action";
import { fillPoolDetails } from "@/servers/actions/pool-request-action";
import { usePoolRequestInfo } from "@/servers/data/use-pool-request-info";
import { Loader2, MapPin, Upload } from "lucide-react";
import { useTranslations } from "next-intl";
import { useEffect, useMemo, useState, useTransition } from "react";
import { useDropzone } from "react-dropzone";
import { Controller } from "react-hook-form";
import { toast } from "sonner";

export function PoolDetailsForm() {
	const t = useTranslations();
	const router = useRouter();
	const { poolRequestInfo, refetch } = usePoolRequestInfo();
	const [isPending, startTransition] = useTransition();
	const [is_submitting, submittingTransition] = useTransition();
	const [isVisiting, visitingTransition] = useTransition();

	const [cities, setCities] = useState<{ value: string; label: string }[]>([]);
	const [areas, setAreas] = useState<{ value: string; label: string }[]>([]);
	const [selectedCity, setSelectedCity] = useState<string | null>(null);
	const [isLoadingAreas, setIsLoadingAreas] = useState(false);
	const [files, setFiles] = useState({
		file: null,
	});
	const [selectedLocation, setSelectedLocation] = useState<{ lat: number; lng: number } | null>(null);
	const [isMapDialogOpen, setIsMapDialogOpen] = useState(false);

	const fields = useMemo(
		() =>
			prepareFields({
				first_name: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: t("FirstName"),
						labelClassName: "text-white",
						placeholder: t("FirstName"),
						containerClassName: "col-span-4",
					}),
				},
				middle_name: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: t("MiddleName"),
						labelClassName: "text-white",
						placeholder: t("MiddleName"),
						containerClassName: "col-span-4",
					}),
				},
				length: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: t("Length"),
						labelClassName: "text-white",
						placeholder: t("Length"),
						containerClassName: "col-span-4",
					}),
				},
				last_name: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: t("LastName"),
						labelClassName: "text-white",
						placeholder: t("LastName"),
						containerClassName: "col-span-4",
					}),
				},
				city_id: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: ({ name, control, label, containerClassName }) => (
						<Controller
							name={name}
							control={control}
							render={({ field, fieldState: { error } }) => {
								console.log("field: ", field.value);
								return (
									<div className={cn("col-span-4 p-2", containerClassName)}>
										<label className="mb-1 block text-base font-medium text-white">
											{label}
										</label>
										<Select
											value={field.value || ""}
											onValueChange={(value) => {
												field.onChange(value);
												setSelectedCity(value);
												FormHooks.setValue("area_id", "");
											}}
										>
											<SelectTrigger className="h-11 w-full rounded-md border-[#DEDEDE] bg-white">
												<SelectValue placeholder={t("Select City")} />
											</SelectTrigger>
											<SelectContent>
												{cities.map((city) => (
													<SelectItem key={city.value} value={city.value}>
														{city.label}
													</SelectItem>
												))}
											</SelectContent>
										</Select>
										{error && (
											<span className="text-sm text-red-500">{error.message}</span>
										)}
									</div>
								);
							}}
						/>
					),
					componentProps: () => ({
						label: t("City"),
						containerClassName: "col-span-4",
					}),
				},
				width: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: t("Width"),
						labelClassName: "text-white",
						placeholder: t("Width"),
						containerClassName: "col-span-4",
					}),
				},
				phone: {
					yup: Yup.string().optional(),
					type: "input",
					component: (props) => (
						<div className="col-span-4">
							<label htmlFor="phone" className="text-white">
								{t("Phone")}
							</label>
							<div className="rounded-md border border-gray-300 bg-white h-11 mt-3 p-2">
								<PhoneInput
									{...props}
									value={poolRequestInfo?.user?.phone?.slice(3)}
									onChange={(value: string | number) => {
										if (props.onChange) {
											props.onChange(value.toString());
										}
									}}
									labelclassName="text-right"
									className="h-11 -mt-2"
									disabled
								/>
							</div>
						</div>
					),
					componentProps: ({ }) => ({
						type: "text",
						labelClassName: "text-white",
						placeholder: t("Phone"),
						containerClassName: "col-span-4",
					}),
				},
				area_id: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: ({ name, control, label, containerClassName }) => (
						<Controller
							name={name}
							control={control}
							render={({ field, fieldState: { error } }) => (
								<div className={cn("col-span-4 p-2", containerClassName)}>
									<label className="mb-1 block text-base font-medium text-white">
										{label}
									</label>
									<Select
										value={field.value || ""}
										onValueChange={field.onChange}
										disabled={!selectedCity || isLoadingAreas}
									>
										<SelectTrigger className="h-11 w-full rounded-md border-[#DEDEDE] bg-white">
											<SelectValue placeholder={t("Select Area")} />
										</SelectTrigger>
										<SelectContent>
											{areas.map((area) => (
												<SelectItem key={area.value} value={area.value}>
													{area.label}
												</SelectItem>
											))}
										</SelectContent>
									</Select>
									{error && (
										<span className="text-sm text-red-500">{error.message}</span>
									)}
								</div>
							)}
						/>
					),
					componentProps: () => ({
						label: t("Area"),
						containerClassName: "col-span-4",
					}),
				},
				depth: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: t("depth"),
						labelClassName: "text-white",
						placeholder: t("depth"),
						containerClassName: "col-span-4",
					}),
				},
				attachment: {
					yup: Yup.mixed().nullable().optional(),
					type: "input",
					component: () => (
						<div className="col-span-4">
							<label htmlFor="file-upload" className="text-white">
								{t("Upload File")}
							</label>
							<div
								{...getRootProps()}
								className="col-span-4 cursor-pointer rounded-md border bg-white border-[#DEDEDE] p-4 mt-[11.2px] h-10"
							>

								<div>
									<input {...getInputProps()} />
									<div className="flex items-center gap-2 -mt-2">
										<Upload className="h-5 w-5 black" />
										<p className="text-[14px] text-neutral-400">
											{
												// eslint-disable-next-line @typescript-eslint/ban-ts-comment
												// @ts-expect-error
												files[name]?.name || t("Upload File")
											}
										</p>
									</div>
								</div>
							</div>
						</div>
					),
					componentProps: ({ }) => ({
						type: "file",
						label: t("Attachment"),
						labelClassName: "text-white",
						placeholder: t("Upload File"),
						containerClassName: "col-span-4",
					}),
				},
				email: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: t("Email"),
						labelClassName: "text-white",
						placeholder: t("Email"),
						containerClassName: "col-span-4",

					}),
				},
				distance_to_pump_room: {
					yup: Yup.string().required(t("This field is required")),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "text",
						label: t("distance_to_pump_room"),
						labelClassName: "text-white",
						placeholder: t("distance_to_pump_room"),
						containerClassName: "col-span-4",
					}),
				},
				location_lat: {
					yup: Yup.string().optional(),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "hidden",
						containerClassName: "hidden",
					}),
				},
				location_lng: {
					yup: Yup.string().optional(),
					type: "input",
					component: InputField,
					componentProps: ({ }) => ({
						type: "hidden",
						containerClassName: "hidden",
					}),
				},
			}),
		[t, cities, areas, isLoadingAreas, selectedCity, selectedLocation]
	);


	const FormHooks = useFormBuilder(fields, {
		defaultValues: {
			first_name: "",
			middle_name: "",
			last_name: "",
			area_id: "",
			city_id: "",
			width: "",
			phone: "",
			depth: "",
			attachment: null,
			distance_to_pump_room: "",
			email: "",
			location_lat: "",
			location_lng: "",
		},
	});


	// Fetch cities on component mount
	useEffect(() => {
		const loadCities = async () => {
			try {
				const citiesData = (await fetchCity())?.data;
				const formattedCities = citiesData?.map((city: any) => ({
					value: city.id.toString(),
					label: t(city.name_en),
				}));
				setCities(formattedCities);
			} catch (error) {
				console.error("Error fetching cities:", error);
				toast.error(t("Error loading cities"));
			}
		};
		loadCities();
	}, []);

	// Fetch areas when city changes
	useEffect(() => {
		const loadAreas = async () => {
			if (!selectedCity) {
				setAreas([]);
				return;
			}

			setIsLoadingAreas(true);
			try {
				const areasData = (await fetchArea(selectedCity))?.data;
				const formattedAreas = areasData?.map((area: any) => ({
					value: area.id.toString(),
					label: t(area.name_en),
				}));
				setAreas(formattedAreas);
			} catch (error) {
				console.error("Error fetching areas:", error);
				toast.error(t("Error loading areas"));
			} finally {
				setIsLoadingAreas(false);
			}
		};
		loadAreas();
	}, [selectedCity, t]);


	// Reset form when poolRequestInfo changes
	useEffect(() => {
		if (poolRequestInfo) {
			const { first_name, middle_name, last_name, area_id, city_id, width, user, depth, attachment, distance_to_pump_room, email, length, latitude, longitude } = poolRequestInfo;
			console.log("city_id: ", city_id);
			console.log("area_id: ", area_id);

			// First set the selected city to trigger area loading
			if (city_id) {
				// Convert city_id to string if it's a number
				const cityIdString = city_id.toString();
				setSelectedCity(cityIdString);
			}

			// Set location if available
			if (latitude && longitude) {
				setSelectedLocation({ lat: parseFloat(latitude), lng: parseFloat(longitude) });
			}

			// Then reset the form with all values
			FormHooks.reset({
				first_name,
				middle_name,
				last_name,
				area_id: area_id ? area_id.toString() : "",
				city_id: city_id ? city_id.toString() : "",
				width,
				phone: user?.phone?.slice(3),
				depth,
				attachment,
				distance_to_pump_room,
				email,
				length,
				location_lat: latitude || "",
				location_lng: longitude || ""
			});
		}
	}, [FormHooks, poolRequestInfo]);

	// Add a new useEffect to set area_id after areas are loaded
	useEffect(() => {
		if (poolRequestInfo?.area_id && areas.length > 0) {
			// Convert area_id to string if it's a number
			const areaIdString = poolRequestInfo.area_id.toString();

			// Check if the area_id exists in the loaded areas
			const areaExists = areas.some(area => area.value === areaIdString);
			if (areaExists) {
				FormHooks.setValue("area_id", areaIdString);
			}
		}
	}, [areas, poolRequestInfo, FormHooks]);


	const onDrop = (name: keyof typeof files) => (acceptedFiles: File[]) => {
		setFiles((prev: any) => ({ ...prev, [name]: acceptedFiles[0] || null }));
	};


	const { getRootProps, getInputProps } = useDropzone({
		onDrop: onDrop("file"),
		accept: {
			"application/pdf": [".pdf"],
			"application/msword": [".doc", ".docx"],
			"image/*": [".jpg", ".jpeg", ".png", ".gif"],
		},
	});

	console.log(FormHooks.watch());
	const handleShapeSelect = () => {
		startTransition(() => {
			// router.push("/simulation-program/design-shape" as any);
			const pool = poolRequestInfo?.design_type === "skimmer" || poolRequestInfo?.design_type === "overflow" ? 1 : 2;
			router.push(`/simulation-program/pool-tiles?pool=${pool}` as any);
		});
	};

	const handleVisitSelect = () => {
		visitingTransition(() => {
			router.push("/simulation-program/request-visit" as any);
		});
	};

	const handleLocationSelect = (lat: number, lng: number) => {
		setSelectedLocation({ lat, lng });
		FormHooks.setValue("location_lat", lat.toString());
		FormHooks.setValue("location_lng", lng.toString());
		setIsMapDialogOpen(false);
	};

	async function onSubmit(data: any) {
		try {
			console.log("Form submission started with data:", data);

			submittingTransition(async () => {
				try {
					const formData = new FormData();
					formData.append("first_name", data.first_name);
					formData.append("middle_name", data.middle_name);
					formData.append("last_name", data.last_name);
					formData.append("email", data.email);
					formData.append("city_id", data.city_id);
					formData.append("area_id", data.area_id);
					formData.append("width", data.width);
					formData.append("depth", data.depth);
					formData.append("length", data.length);
					formData.append("distance_to_pump_room", data.distance_to_pump_room);
					formData.append("latitude", data.latitude);
					formData.append("longitude", data.longitude);

					// Explicitly set attachment to null if not provided
					formData.append("attachment", "null");

					// If a file was uploaded, append it
					/*if (files.file) {
						formData.append("attachment", files.file);
					}*/

					console.log("Sending form data to API...");
					console.log("FormData entries:");
					for (const [key, value] of formData.entries()) {
						console.log(`${key}: ${value}`);
					}

					const result = await fillPoolDetails(formData);
					console.log("API response:", result);

					if (result?.success) {
						/*toast.success(t("Request submitted successfully"), {
							style: {
								backgroundColor: "green",
								color: "white",
							},
						});*/

						await refetch();
					} else {
						console.error("API error:", result);
						if (Array.isArray(result?.message)) {
							result?.message?.map((error: any) => {
								console.error("API error message:", error);
								toast.error(t(error.replace(".", "")));
							});
						}
						else {
							console.error("API error message:", result?.message);
							toast.error(t(result?.message));
						}
					}
				} catch (innerError) {
					console.error("Error inside startTransition:", innerError);
					toast.error(t("Error submitting form"));
				}
			});
		} catch (error) {
			console.error("Registration error:", error);
			toast.error(t("Something went wrong"));
		}
	}

	return (
		<form onSubmit={FormHooks.handleSubmit(onSubmit, (errors) => {
			console.log("Form validation errors:", errors);

			if (Object.keys(errors).length > 0) {
				const errorMessages = Object.keys(errors).map(key => {
					return `${t(key)}: ${errors[key as keyof typeof errors]?.message}`;
				});

				console.log("Error messages:", errorMessages);

				toast.error(t("Please fix the following errors"), {
					style: {
						backgroundColor: "red",
						color: "white",
					},
				});
			}
		})}>
			<div className="flex flex-col items-center justify-center p-4">
				<FormBuilder
					fields={fields}
					FormHooks={FormHooks}
					className="grid grid-cols-12 p-2"
				/>

				{/* Map Dialog */}
				<div className="col-span-12 mb-6">
					<div className="mt-2">
						<Dialog open={isMapDialogOpen} onOpenChange={setIsMapDialogOpen}>
							<DialogTrigger asChild>
								<Button
									type="button"
									variant="outline"
									className="w-full h-12 text-lg text-center text-white py-3 bg-gradient-to-r from-[#1E1E1E] to-[rgba(30,30,30,0.7)] rounded-full transition-transform duration-300 ease-in-out hover:scale-105 hover:shadow-xl focus:outline-none focus:ring-4 focus:ring-gray-600 gap-2"
								>
									<MapPin className="h-4 w-4" />
									{selectedLocation
										? `${selectedLocation.lat.toFixed(6)}, ${selectedLocation.lng.toFixed(6)}`
										: t("Select Your Location")}
								</Button>
							</DialogTrigger>
							<DialogContent className="sm:max-w-[600px]">
								<DialogHeader>
									<DialogTitle>{t("Select Your Location")}</DialogTitle>
								</DialogHeader>
								<div className="h-[400px] w-full">
									<Map
										onLocationSelect={handleLocationSelect}
										initialPosition={selectedLocation ? [selectedLocation.lat, selectedLocation.lng] : undefined}
									/>
								</div>
							</DialogContent>
						</Dialog>
					</div>
				</div>

				<div className="flex w-full items-center justify-center gap-4">
					<button
						type="submit"
						className="w-32 h-12 text-lg text-center text-white py-2 bg-gradient-to-r from-[#1E1E1E] to-[rgba(30,30,30,0.7)] rounded-full transition-transform duration-300 ease-in-out hover:scale-105 hover:shadow-xl focus:outline-none focus:ring-4 focus:ring-gray-600"
						disabled={is_submitting}
					>
						{is_submitting ? <Loader2 className="mx-auto w-4 h-4 animate-spin" /> : t("Save")}
					</button>
				</div>
				<span className="text-sm text-white my-4">في حال لم يكن لديك معلومات يمكنك طلب زيارة ميدانية ليقوم فريقنا بأخذ المقاسات</span>
				<div className="flex w-full items-center justify-center gap-4">
					<button
						onClick={handleVisitSelect}
						className="w-32 h-12 text-lg text-center text-white py-2 bg-gradient-to-r from-[#1E1E1E] to-[rgba(30,30,30,0.7)] rounded-full transition-transform duration-300 ease-in-out hover:scale-105 hover:shadow-xl focus:outline-none focus:ring-4 focus:ring-gray-600"
						disabled={isVisiting}
					>
						{isVisiting ? <Loader2 className="mx-auto w-4 h-4 animate-spin" /> : "طلب زيارة"}
					</button>
				</div>
				<FormButtons
					onPrevious={() => router.back()}
					onNext={handleShapeSelect}
					isPending={isPending}
					classname="-mb-20 -mr-52"
				/>
			</div>
		</form>
	);
} 